#include "matrix.h"
#include "math.h"

/* dot(x,y)
 * Returns dot product of x and y
 */
double dot(double *x, double *y)
{
    return (x[0]*y[0] + x[1]*y[1] + x[2]*y[2]);
}

/* getSTAR(z, x)
 * Calculate cross product matrix of x
*/
void getSTAR(double *z, double *x)
{
    z[0] = 0;
    z[1] = x[2];
    z[2] = -x[1];
    z[3] = -x[2];
    z[4] = 0;
    z[5] = x[0];
    z[6] = x[1];
    z[7] = -x[0];
    z[8] = 0;
}

/* cross(z, x, y)
* Calculate cross product between x and y, result is stored in z
*/
void cross(double *z, double *x, double *y)
{
    z[0] = x[1]*y[2] - x[2]*y[1];
    z[1] = x[2]*y[0] - x[0]*y[2];
    z[2] = x[0]*y[1] - x[1]*y[0];
}

/* double norm(double *x)
* Returns the norm of vector x
*/
double norm(double *x)
{
    return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);
}


/* [phi, Gt, F_ext] = getConstraints(NP, NC, NCPP, U_0, L, Ui, Uj, Uk, pEdgeNext, pEdgePrev, blue, red, q, eqAngi, eqAngj, eqAngk, trap, Uold)
* Calculates constraint, forces and jacobian.

* Parameters:
* NP: Number of blocks
* NC: Number of constraints
* NCPP: Number of constraints per block
* U_0: Depth of LJ potential
* L: Characteristic length of LJ potential
* Ui,Uj,Uk: Body axes
* pEdgeNext, pEdgePrev: Next and previous edgepoints on blocks
* blue, red: Position of red/blue charge on blocks
* q: Generalized position vector
* eqAngi, eqAngj, eqAngk: Equilibrium angles
* trap: The position of the trap

* Returns:
* phi: Constraints
* Gt: Transpose of constraint jacobian
* F_ext: External forces (LJ)
*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Input variables */
    int NP;
    int NC;
    int NCPP;
    double U_0;
    double L;
    const double *Ui, *Uj, *Uk;
    const double *pEdgeNext;
    const double *pEdgePrev;
    const double *blue;
    const double *red;
    const double *q;
    const double *eqAngi, *eqAngj, *eqAngk;
    const double *trap;
    const double *Uold;
    /* Output variables */
    double *phi;
    double *Gt;
    int *Gt_ir;
    int *Gt_jc;
    double *F_ext;
    /* Others */
    int i;
    int p = 0;
    int tmp1 = 0;
    int tmp2 = 0;
    double tmp3[3];
    double di_dot;
    double dj_dot;
    double dk_dot;
    double di_x[3];
    double dj_x[3];
    double dk_x[3];
    double theta1;
    double theta2;
    double theta3;
    double r[3];
    double F[3];
    double F_abs;
    double Fp[3];
    double pxF[3];
    double L2;
    double edgeNextSTAR[9];
    double edgePrevSTAR[9];
    double A[3];
    double B[3];
    double C[3];
    double A_abs;
    double B_abs;
    double C_abs;
    double eps;
    
    
    /* Check for correct number of parameters */
    if (nlhs != 3 || nrhs != 18)
        mexErrMsgTxt("Wrong number of parameters");
        
    /* Get input pointers */
    NP = *mxGetPr(prhs[0]);
    NC = *mxGetPr(prhs[1]);
    NCPP = *mxGetPr(prhs[2]);
    U_0 = *mxGetPr(prhs[3]);
    L = *mxGetPr(prhs[4]);
    Ui = mxGetPr(prhs[5]);
    Uj = mxGetPr(prhs[6]);
    Uk = mxGetPr(prhs[7]);
    pEdgeNext = mxGetPr(prhs[8]);
    pEdgePrev = mxGetPr(prhs[9]);
    blue = mxGetPr(prhs[10]);
    red = mxGetPr(prhs[11]);
    q = mxGetPr(prhs[12]);
    eqAngi = mxGetPr(prhs[13]);
    eqAngj = mxGetPr(prhs[14]);
    eqAngk = mxGetPr(prhs[15]);
    trap = mxGetPr(prhs[16]);
    Uold = mxGetPr(prhs[17]);

    
    /* Reserve memory */
    plhs[0] = mxCreateDoubleMatrix(NC, 1, mxREAL); /* phi */
    plhs[1] = mxCreateSparse(6*NP, NC, 42*(NP-1)+21+12, mxREAL); /* Gt */
    plhs[2] = mxCreateDoubleMatrix(6*NP, 1, mxREAL); /* F_ext */
    
    /* Get pointers to output parameters */
    phi = mxGetPr(plhs[0]);
    Gt = mxGetPr(plhs[1]);
    Gt_ir = mxGetIr(plhs[1]);
    Gt_jc = mxGetJc(plhs[1]);
    F_ext = mxGetPr(plhs[2]);
    
    eps = mxGetEps();
    
    /* Main loop */
    for (p = 0; p < NP-1; p++)
    {
        /* Calculate phi */
        tmp1 = 3*p;
        tmp2 = 3*(p+1);
        di_dot = Ui[tmp1]*Ui[tmp2] + Ui[tmp1+1]*Ui[tmp2+1] + Ui[tmp1+2]*Ui[tmp2+2];
        dj_dot = Uj[tmp1]*Uj[tmp2] + Uj[tmp1+1]*Uj[tmp2+1] + Uj[tmp1+2]*Uj[tmp2+2];
        dk_dot = Uk[tmp1]*Uk[tmp2] + Uk[tmp1+1]*Uk[tmp2+1] + Uk[tmp1+2]*Uk[tmp2+2];
        cross(di_x, Ui+tmp1, Ui+tmp2);
        cross(dj_x, Uj+tmp1, Uj+tmp2);
        cross(dk_x, Uk+tmp1, Uk+tmp2);
        
        theta1 = atan2(norm(di_x), di_dot);
        theta2 = atan2(norm(dk_x), dk_dot);
        theta3 = atan2(norm(dj_x), dj_dot);
        
        tmp2 = 7*p;
        phi[NCPP*p] = q[tmp2] + pEdgeNext[tmp1] - q[7*(p+1)] - pEdgePrev[3*(p+1)];
        phi[NCPP*p+1] = q[tmp2+1] + pEdgeNext[tmp1+1] - q[7*(p+1)+1] - pEdgePrev[3*(p+1)+1];
        phi[NCPP*p+2] = q[tmp2+2] + pEdgeNext[tmp1+2] - q[7*(p+1)+2] - pEdgePrev[3*(p+1)+2];
        phi[NCPP*p+3] = theta1 - eqAngi[p];
        phi[NCPP*p+4] = theta2 - eqAngk[p];
        phi[NCPP*p+5] = theta3 - eqAngj[p];
        
        /* Calculate transpose of Jacobian */
        getSTAR(edgeNextSTAR, pEdgeNext+3*p);
        getSTAR(edgePrevSTAR, pEdgePrev+3*(p+1));
        A_abs = -1/(dot(di_x,di_x) + di_dot*di_dot)*(norm(di_x)+di_dot*di_dot/(norm(di_x)+eps));
        B_abs = -1/(dot(dk_x,dk_x) + dk_dot*dk_dot)*(norm(dk_x)+dk_dot*dk_dot/(norm(dk_x)+eps));
        C_abs = -1/(dot(dj_x,dj_x) + dj_dot*dj_dot)*(norm(dj_x)+dj_dot*dj_dot/(norm(dj_x)+eps));
        A[0] = A_abs*di_x[0];
        A[1] = A_abs*di_x[1];
        A[2] = A_abs*di_x[2];
        B[0] = B_abs*dk_x[0];
        B[1] = B_abs*dk_x[1];
        B[2] = B_abs*dk_x[2];
        C[0] = C_abs*dj_x[0];
        C[1] = C_abs*dj_x[1];
        C[2] = C_abs*dj_x[2];
        
        tmp1 = 42*p;
        Gt[tmp1] = 1.;
        Gt[tmp1+1] = -edgeNextSTAR[0];
        Gt[tmp1+2] = -edgeNextSTAR[3];
        Gt[tmp1+3] = -edgeNextSTAR[6];
        Gt[tmp1+4] = -1.;
        Gt[tmp1+5] = edgePrevSTAR[0];
        Gt[tmp1+6] = edgePrevSTAR[3];
        Gt[tmp1+7] = edgePrevSTAR[6];
        
        Gt[tmp1+8] = 1.;
        Gt[tmp1+9] = -edgeNextSTAR[1];
        Gt[tmp1+10] = -edgeNextSTAR[4];
        Gt[tmp1+11] = -edgeNextSTAR[7];
        Gt[tmp1+12] = -1.;
        Gt[tmp1+13] = edgePrevSTAR[1];
        Gt[tmp1+14] = edgePrevSTAR[4];
        Gt[tmp1+15] = edgePrevSTAR[7];
        
        Gt[tmp1+16] = 1.;
        Gt[tmp1+17] = -edgeNextSTAR[2];
        Gt[tmp1+18] = -edgeNextSTAR[5];
        Gt[tmp1+19] = -edgeNextSTAR[8];
        Gt[tmp1+20] = -1.;
        Gt[tmp1+21] = edgePrevSTAR[2];
        Gt[tmp1+22] = edgePrevSTAR[5];
        Gt[tmp1+23] = edgePrevSTAR[8];
        
        Gt[tmp1+24] = A[0];
        Gt[tmp1+25] = A[1];
        Gt[tmp1+26] = A[2];
        Gt[tmp1+27] = -A[0];
        Gt[tmp1+28] = -A[1];
        Gt[tmp1+29] = -A[2];
        
        Gt[tmp1+30] = B[0];
        Gt[tmp1+31] = B[1];
        Gt[tmp1+32] = B[2];
        Gt[tmp1+33] = -B[0];
        Gt[tmp1+34] = -B[1];
        Gt[tmp1+35] = -B[2];
        
        Gt[tmp1+36] = C[0];
        Gt[tmp1+37] = C[1];
        Gt[tmp1+38] = C[2];
        Gt[tmp1+39] = -C[0];
        Gt[tmp1+40] = -C[1];
        Gt[tmp1+41] = -C[2];
        
        /* Set Ir part of Gt */
        tmp2 = 6*p;
        Gt_ir[tmp1] = tmp2;
        Gt_ir[tmp1+1] = tmp2+3;
        Gt_ir[tmp1+2] = tmp2+4;
        Gt_ir[tmp1+3] = tmp2+5;
        Gt_ir[tmp1+4] = tmp2+6;
        Gt_ir[tmp1+5] = tmp2+9;
        Gt_ir[tmp1+6] = tmp2+10;
        Gt_ir[tmp1+7] = tmp2+11;
        
        Gt_ir[tmp1+8] = tmp2+1;
        Gt_ir[tmp1+9] = tmp2+3;
        Gt_ir[tmp1+10] = tmp2+4;
        Gt_ir[tmp1+11] = tmp2+5;
        Gt_ir[tmp1+12] = tmp2+7;
        Gt_ir[tmp1+13] = tmp2+9;
        Gt_ir[tmp1+14] = tmp2+10;
        Gt_ir[tmp1+15] = tmp2+11;
        
        Gt_ir[tmp1+16] = tmp2+2;
        Gt_ir[tmp1+17] = tmp2+3;
        Gt_ir[tmp1+18] = tmp2+4;
        Gt_ir[tmp1+19] = tmp2+5;
        Gt_ir[tmp1+20] = tmp2+8;
        Gt_ir[tmp1+21] = tmp2+9;
        Gt_ir[tmp1+22] = tmp2+10;
        Gt_ir[tmp1+23] = tmp2+11;
        
        Gt_ir[tmp1+24] = tmp2+3;
        Gt_ir[tmp1+25] = tmp2+4;
        Gt_ir[tmp1+26] = tmp2+5;
        Gt_ir[tmp1+27] = tmp2+9;
        Gt_ir[tmp1+28] = tmp2+10;
        Gt_ir[tmp1+29] = tmp2+11;
        
        Gt_ir[tmp1+30] = tmp2+3;
        Gt_ir[tmp1+31] = tmp2+4;
        Gt_ir[tmp1+32] = tmp2+5;
        Gt_ir[tmp1+33] = tmp2+9;
        Gt_ir[tmp1+34] = tmp2+10;
        Gt_ir[tmp1+35] = tmp2+11;
        
        Gt_ir[tmp1+36] = tmp2+3;
        Gt_ir[tmp1+37] = tmp2+4;
        Gt_ir[tmp1+38] = tmp2+5;
        Gt_ir[tmp1+39] = tmp2+9;
        Gt_ir[tmp1+40] = tmp2+10;
        Gt_ir[tmp1+41] = tmp2+11;
        
        /* Set Jc part of Gt */
        tmp2 = 6*p;
        Gt_jc[tmp2] = tmp1;
        Gt_jc[tmp2+1] = tmp1+8;
        Gt_jc[tmp2+2] = tmp1+16;
        Gt_jc[tmp2+3] = tmp1+24;
        Gt_jc[tmp2+4] = tmp1+30;
        Gt_jc[tmp2+5] = tmp1+36;
    }

    /* Set phi for last block */
    p = NP-1;

    phi[NCPP*p] = q[7*p];
    phi[NCPP*p+1] = q[7*p+1];
    phi[NCPP*p+2] = q[7*p+2];
    phi[NCPP*p+3] = Ui[3*p]*Uold[0] + Ui[3*p+1]*Uold[1] + Ui[3*p+2]*Uold[2];
    phi[NCPP*p+4] = Ui[3*p]*Uold[3] + Ui[3*p+1]*Uold[4] + Ui[3*p+2]*Uold[5];
    phi[NCPP*p+5] = Uk[3*p]*Uold[0] + Uk[3*p+1]*Uold[1] + Uk[3*p+2]*Uold[2];
    
    /* Set Gt for last block */
    p = NP-1;
    tmp3[0] = tmp3[1] = 0;
    tmp3[2] = 1.;
    cross(A, Ui+3*p, Uold);
    cross(C, Uk+3*p, Uold);
    tmp3[2] = 0;
    tmp3[1] = 1.;
    cross(B, Ui+3*p, Uold+3);
    getSTAR(edgeNextSTAR, pEdgeNext+3*p);
    
    tmp1 = 42*p;
    Gt[tmp1] = 1.;
    Gt[tmp1+1] = 0; /*-edgeNextSTAR[0];*/
    Gt[tmp1+2] = 0; /*-edgeNextSTAR[3];*/
    Gt[tmp1+3] = 0; /*-edgeNextSTAR[6];*/
    
    Gt[tmp1+4] = 1.;
    Gt[tmp1+5] = 0;/*-edgeNextSTAR[1];*/
    Gt[tmp1+6] = 0;/*-edgeNextSTAR[4];*/
    Gt[tmp1+7] = 0;/*-edgeNextSTAR[7];*/
    
    Gt[tmp1+8] = 1.;
    Gt[tmp1+9] = 0;/*-edgeNextSTAR[2];*/
    Gt[tmp1+10] = 0;/*-edgeNextSTAR[5];*/
    Gt[tmp1+11] = 0;/*-edgeNextSTAR[8];*/
    
    Gt[tmp1+12] = A[0];
    Gt[tmp1+13] = A[1];
    Gt[tmp1+14] = A[2];
    
    Gt[tmp1+15] = B[0];
    Gt[tmp1+16] = B[1];
    Gt[tmp1+17] = B[2];
    
    Gt[tmp1+18] = C[0];
    Gt[tmp1+19] = C[1];
    Gt[tmp1+20] = C[2];
    
    /* Set Ir part of Gt */
    tmp2 = 6*p;
    Gt_ir[tmp1] = tmp2;
    Gt_ir[tmp1+1] = tmp2+3;
    Gt_ir[tmp1+2] = tmp2+4;
    Gt_ir[tmp1+3] = tmp2+5;
    
    Gt_ir[tmp1+4] = tmp2+1;
    Gt_ir[tmp1+5] = tmp2+3;
    Gt_ir[tmp1+6] = tmp2+4;
    Gt_ir[tmp1+7] = tmp2+5;
    
    Gt_ir[tmp1+8] = tmp2+2;
    Gt_ir[tmp1+9] = tmp2+3;
    Gt_ir[tmp1+10] = tmp2+4;
    Gt_ir[tmp1+11] = tmp2+5;
    
    Gt_ir[tmp1+12] = tmp2+3;
    Gt_ir[tmp1+13] = tmp2+4;
    Gt_ir[tmp1+14] = tmp2+5;
    
    Gt_ir[tmp1+15] = tmp2+3;
    Gt_ir[tmp1+16] = tmp2+4;
    Gt_ir[tmp1+17] = tmp2+5;
    
    Gt_ir[tmp1+18] = tmp2+3;
    Gt_ir[tmp1+19] = tmp2+4;
    Gt_ir[tmp1+20] = tmp2+5;
    
    /* Set Jc part of Gt */
    tmp2 = 6*p;
    Gt_jc[tmp2] = tmp1;
    Gt_jc[tmp2+1] = tmp1+4;
    Gt_jc[tmp2+2] = tmp1+8;
    Gt_jc[tmp2+3] = tmp1+12;
    Gt_jc[tmp2+4] = tmp1+15;
    Gt_jc[tmp2+5] = tmp1+18;
    
    /* Get trap constraint */
    phi[NC-3] = red[0] - trap[0];
    phi[NC-2] = red[1] - trap[1];
    phi[NC-1] = red[2] - trap[2];
    
    tmp3[0] = red[0] - q[0];
    tmp3[1] = red[1] - q[1];
    tmp3[2] = red[2] - q[2];
    getSTAR(edgePrevSTAR, tmp3);
    Gt[tmp1+21] = 1.;
    Gt[tmp1+22] = -edgePrevSTAR[0];
    Gt[tmp1+23] = -edgePrevSTAR[3];
    Gt[tmp1+24] = -edgePrevSTAR[6];
    
    Gt[tmp1+25] = 1.;
    Gt[tmp1+26] = -edgePrevSTAR[1];
    Gt[tmp1+27] = -edgePrevSTAR[4];
    Gt[tmp1+28] = -edgePrevSTAR[7];
    
    Gt[tmp1+29] = 1.;
    Gt[tmp1+30] = -edgePrevSTAR[2];
    Gt[tmp1+31] = -edgePrevSTAR[5];
    Gt[tmp1+32] = -edgePrevSTAR[8];
    
    /* Set Ir part of trap constraint */
    Gt_ir[tmp1+21] = 0;
    Gt_ir[tmp1+22] = 3;
    Gt_ir[tmp1+23] = 4;
    Gt_ir[tmp1+24] = 5;
    
    Gt_ir[tmp1+25] = 1;
    Gt_ir[tmp1+26] = 3;
    Gt_ir[tmp1+27] = 4;
    Gt_ir[tmp1+28] = 5;
    
    Gt_ir[tmp1+29] = 2;
    Gt_ir[tmp1+30] = 3;
    Gt_ir[tmp1+31] = 4;
    Gt_ir[tmp1+32] = 5;
    
    /* Set Jc part of trap constraint */
    tmp2 = NC-3;
    Gt_jc[tmp2] = tmp1+21;
    Gt_jc[tmp2+1] = tmp1+25;
    Gt_jc[tmp2+2] = tmp1+29;
    
    Gt_jc[tmp2+3] = tmp1+33;
    
    /* Calculate external Force */
    L2 = L*L;
    for (p = 0; p < NP-3; p++)
    {
        r[0] = blue[3*p] - red[3*(p+3)];
        r[1] = blue[3*p+1] - red[3*(p+3)+1];
        r[2] = blue[3*p+2] - red[3*(p+3)+2];
        
        F_abs = -2.*U_0*exp(-(r[0]*r[0]+r[1]*r[1]+r[2]*r[2])/L2)/L2;
        F[0] = r[0]*F_abs;
        F[1] = r[1]*F_abs;
        F[2] = r[2]*F_abs;
        
        F_ext[6*p] = F_ext[6*p] + F[0];
        F_ext[6*p+1] = F_ext[6*p+1] + F[1];
        F_ext[6*p+2] = F_ext[6*p+2] + F[2];
        F_ext[6*(p+3)] = F_ext[6*(p+3)] - F[0];
        F_ext[6*(p+3)+1] = F_ext[6*(p+3)+1] - F[1];
        F_ext[6*(p+3)+2] = F_ext[6*(p+3)+2] - F[2];
        
        Fp[0] = blue[3*p] - q[7*p];
        Fp[1] = blue[3*p+1] - q[7*p+1];
        Fp[2] = blue[3*p+2] - q[7*p+2];
        cross(pxF, Fp, F);
        F_ext[6*p+3] = F_ext[6*p+3] + pxF[0];
        F_ext[6*p+4] = F_ext[6*p+4] + pxF[1];
        F_ext[6*p+5] = F_ext[6*p+5] + pxF[2];
        
        Fp[0] = red[3*(p+3)] - q[7*(p+3)];
        Fp[1] = red[3*(p+3)+1] - q[7*(p+3)+1];
        Fp[2] = red[3*(p+3)+2] - q[7*(p+3)+2];
        cross(pxF, Fp, F);
        F_ext[6*(p+3)+3] = F_ext[6*(p+3)+3] - pxF[0];
        F_ext[6*(p+3)+4] = F_ext[6*(p+3)+4] - pxF[1];
        F_ext[6*(p+3)+5] = F_ext[6*(p+3)+5] - pxF[2];
    }
}
