#include "matrix.h"

/*
 * [M, M_inv, M_dot, T, pEdgeNext, pEdgePrev, Ui, Uj, Uk, red, blue] =
 *   updateBlocks(NP, q, v, base, I, I_inv, red_body, blue_body, m);
 * Updates block properties from the provided parameters
 *
 * Parameters:
 * NP: Number of blocks
 * q: Generalized position of blocks
 * v: Generalized velocity of blocks
 * base: Length of blocks
 * I: The general inertia tensor
 * I_inv: General inverse inertia tensor
 * red_body: Positions of red charges in body coordinates
 * blue_body: Position of blue charges in body coordinates
 * m: Mass of blocks
 *
 * Returns:
 * M: Mass matrix
 * M_inv: Inverse mass matrix
 * M_dot: Time derivative of mass matrix
 * T: 
 * pEdgeNext: Position of forward edgepoint
 * pEdgePrev: Position of backward edgepoint
 * Ui, Uj, Uk: Local body axes
 * red: Position of red charges
 * blue: Position of blue charges
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Input parameters */
    int NP;
    const double *q;
    const double *v;
    double base;
    const double *I;
    const double *I_inv;
    const double *red_body;
    const double *blue_body;
    double m;
    /* Output arguments */
    double *M;
    int *M_ir;
    int *M_jc;
    double *M_inv;
    int *M_inv_ir;
    int *M_inv_jc;
    double *M_dot;
    int *M_dot_ir;
    int *M_dot_jc;
    double *T;
    int *T_ir;
    int *T_jc;
    double *pEdgeNext;
    double *pEdgePrev;
    double *Ui, *Uj, *Uk;
    double *red, *blue;
    /* Other variables */
    int p = 0; /* Current particle */
    int col;
    int row;
    int temp1, temp2;
    double m_inv; /* inverse of mass */
    double omegaSTAR[9]; /* cross product matrix of omega */
    double tmpq[3]; /* Used in calculating rotations */
    double qq;
    double qx;
    
    /* Check number of arguments */
    if (nlhs != 11 || nrhs != 9)
        mexErrMsgTxt("Wrong number of input arguments");
    
    
    /* Get pointers to input parameters */
    NP = *mxGetPr(prhs[0]);
    q = mxGetPr(prhs[1]);
    v = mxGetPr(prhs[2]);
    base = *mxGetPr(prhs[3]);
    I = mxGetPr(prhs[4]);
    I_inv = mxGetPr(prhs[5]);
    red_body = mxGetPr(prhs[6]);
    blue_body = mxGetPr(prhs[7]);
    m = *mxGetPr(prhs[8]);
    
    /* Create output matrices */
    plhs[0] = mxCreateSparse(6*NP, 6*NP, 12*NP, mxREAL); /* M */
    plhs[1] = mxCreateSparse(6*NP, 6*NP, 12*NP, mxREAL); /* M_inv */
    plhs[2] = mxCreateSparse(6*NP, 6*NP, 9*NP, mxREAL); /* M_dot */
    plhs[3] = mxCreateSparse(7*NP, 6*NP, 15*NP, mxREAL); /* T */
    plhs[4] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* pEdgeNext */
    plhs[5] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* pEdgePrev */
    plhs[6] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* Ui */
    plhs[7] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* Uj */
    plhs[8] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* Uk */
    plhs[9] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* red */
    plhs[10] = mxCreateDoubleMatrix(3*NP, 1, mxREAL); /* blue */
    
    /* Get pointers to output matrices */
    M = mxGetPr(plhs[0]);
    M_ir = mxGetIr(plhs[0]);
    M_jc = mxGetJc(plhs[0]);
    M_inv = mxGetPr(plhs[1]);
    M_inv_ir = mxGetIr(plhs[1]);
    M_inv_jc = mxGetJc(plhs[1]);
    M_dot = mxGetPr(plhs[2]);
    M_dot_ir = mxGetIr(plhs[2]);
    M_dot_jc = mxGetJc(plhs[2]);
    T = mxGetPr(plhs[3]);
    T_ir = mxGetIr(plhs[3]);
    T_jc = mxGetJc(plhs[3]);
    pEdgeNext = mxGetPr(plhs[4]);
    pEdgePrev = mxGetPr(plhs[5]);
    Ui = mxGetPr(plhs[6]);
    Uj = mxGetPr(plhs[7]);
    Uk = mxGetPr(plhs[8]);
    red = mxGetPr(plhs[9]);
    blue = mxGetPr(plhs[10]);
    
    m_inv = 1/m;
    
    /* Initialize jc parts */
    M_dot_jc[0] = 0;
    M_dot_jc[1] = 0;
    M_dot_jc[2] = 0;
    
    /* Main loop */
    for (p = 0; p < NP; p++)
    {
        
        /* Set mass matrix */
        temp1 = 12*p;
        temp2 = 9*p;
        M[temp1] = m;
        M[temp1+1] = m;
        M[temp1+2] = m;
        M[temp1+3] = I[temp2];
        M[temp1+4] = I[temp2+1];
        M[temp1+5] = I[temp2+2];
        M[temp1+6] = I[temp2+3];
        M[temp1+7] = I[temp2+4];
        M[temp1+8] = I[temp2+5];
        M[temp1+9] = I[temp2+6];
        M[temp1+10] = I[temp2+7];
        M[temp1+11] = I[temp2+8];
        
        /* Set M_ir */
        M_ir[temp1] = 6*p;
        M_ir[temp1+1] = 6*p+1;
        M_ir[temp1+2] = 6*p+2;
        M_ir[temp1+3] = 6*p+3;
        M_ir[temp1+4] = 6*p+4;
        M_ir[temp1+5] = 6*p+5;
        M_ir[temp1+6] = 6*p+3;
        M_ir[temp1+7] = 6*p+4;
        M_ir[temp1+8] = 6*p+5;
        M_ir[temp1+9] = 6*p+3;
        M_ir[temp1+10] = 6*p+4;
        M_ir[temp1+11] = 6*p+5;
        
        /* Set M_jc */
        M_jc[6*p] = temp1;
        M_jc[6*p+1] = temp1+1;
        M_jc[6*p+2] = temp1+2;
        M_jc[6*p+3] = temp1+3;
        M_jc[6*p+4] = temp1+6;
        M_jc[6*p+5] = temp1+9;
        
        /* Set inverse mass matrix */
        temp1 = 12*p;
        temp2 = 9*p;
        M_inv[temp1] = m_inv;
        M_inv[temp1+1] = m_inv;
        M_inv[temp1+2] = m_inv;
        M_inv[temp1+3] = I_inv[temp2];
        M_inv[temp1+4] = I_inv[temp2+1];
        M_inv[temp1+5] = I_inv[temp2+2];
        M_inv[temp1+6] = I_inv[temp2+3];
        M_inv[temp1+7] = I_inv[temp2+4];
        M_inv[temp1+8] = I_inv[temp2+5];
        M_inv[temp1+9] = I_inv[temp2+6];
        M_inv[temp1+10] = I_inv[temp2+7];
        M_inv[temp1+11] = I_inv[temp2+8];
        
        /* Set M_inv_ir */
        M_inv_ir[temp1] = 6*p;
        M_inv_ir[temp1+1] = 6*p+1;
        M_inv_ir[temp1+2] = 6*p+2;
        M_inv_ir[temp1+3] = 6*p+3;
        M_inv_ir[temp1+4] = 6*p+4;
        M_inv_ir[temp1+5] = 6*p+5;
        M_inv_ir[temp1+6] = 6*p+3;
        M_inv_ir[temp1+7] = 6*p+4;
        M_inv_ir[temp1+8] = 6*p+5;
        M_inv_ir[temp1+9] = 6*p+3;
        M_inv_ir[temp1+10] = 6*p+4;
        M_inv_ir[temp1+11] = 6*p+5;
        
        /* Set M_inv_jc */
        M_inv_jc[6*p] = temp1;
        M_inv_jc[6*p+1] = temp1+1;
        M_inv_jc[6*p+2] = temp1+2;
        M_inv_jc[6*p+3] = temp1+3;
        M_inv_jc[6*p+4] = temp1+6;
        M_inv_jc[6*p+5] = temp1+9;
        
        /* Calculate omegaSTAR */
        temp1 = 6*p;
        omegaSTAR[0] = 0;
        omegaSTAR[1] = v[temp1+5];
        omegaSTAR[2] = -v[temp1+4];
        omegaSTAR[3] = -v[temp1+5];
        omegaSTAR[4] = 0;
        omegaSTAR[5] = v[temp1+3];
        omegaSTAR[6] = v[temp1+4];
        omegaSTAR[7] = -v[temp1+3];
        
        /* Calculate M_dot */
        temp1 = 9*p;
        for (col = 0; col < 3; col++)
        {
            for (row = 0; row < 3; row++)
            {
                M_dot[temp1+col*3+row] = omegaSTAR[row]*I[temp1+col*3] + omegaSTAR[row+3]*I[temp1+col*3+1] + omegaSTAR[row+6]*I[temp1+col*3+2];
                M_dot[temp1+col*3+row] = M_dot[temp1+col*3+row] + I[temp1+row]*omegaSTAR[col*3] + omegaSTAR[col*3+1]*I[temp1+row+3] + I[temp1+row+6]*omegaSTAR[col*3+2];
            }
        }
        /* Set M_dot_ir */
        M_dot_ir[temp1] = temp1+3;
        M_dot_ir[temp1+1] = temp1+4;
        M_dot_ir[temp1+2] = temp1+5;
        M_dot_ir[temp1+3] = temp1+3;
        M_dot_ir[temp1+4] = temp1+4;
        M_dot_ir[temp1+5] = temp1+5;
        M_dot_ir[temp1+6] = temp1+3;
        M_dot_ir[temp1+7] = temp1+4;
        M_dot_ir[temp1+8] = temp1+5;
        /* Set M_dot_jc */
        temp2 = 6*p;
        M_dot_jc[temp2+3] = temp1;
        M_dot_jc[temp2+4] = temp1+3;
        M_dot_jc[temp2+5] = temp1+6;
        M_dot_jc[temp2+6] = temp1+9;
        M_dot_jc[temp2+7] = temp1+9;
        M_dot_jc[temp2+8] = temp1+9;
        
        /* Calculate T */
        temp1 = 15*p;
        temp2 = 7*p+3;
        T[temp1] = 1;
        T[temp1+1] = 1;
        T[temp1+2] = 1;
        T[temp1+3] = -0.5*q[temp2+1];
        T[temp1+4] = 0.5*q[temp2];
        T[temp1+5] = -0.5*q[temp2+3];
        T[temp1+6] = 0.5*q[temp2+2];
        T[temp1+7] = -0.5*q[temp2+2];
        T[temp1+8] = 0.5*q[temp2+3];
        T[temp1+9] = 0.5*q[temp2];
        T[temp1+10] = -0.5*q[temp2+1];
        T[temp1+11] = -0.5*q[temp2+3];
        T[temp1+12] = -0.5*q[temp2+2];
        T[temp1+13] = 0.5*q[temp2+1];
        T[temp1+14] = 0.5*q[temp2];
        
        /* Set T_ir */
        temp2 = 7*p;
        T_ir[temp1] = temp2;
        T_ir[temp1+1] = temp2+1;
        T_ir[temp1+2] = temp2+2;
        T_ir[temp1+3] = temp2+3;
        T_ir[temp1+4] = temp2+4;
        T_ir[temp1+5] = temp2+5;
        T_ir[temp1+6] = temp2+6;
        T_ir[temp1+7] = temp2+3;
        T_ir[temp1+8] = temp2+4;
        T_ir[temp1+9] = temp2+5;
        T_ir[temp1+10] = temp2+6;
        T_ir[temp1+11] = temp2+3;
        T_ir[temp1+12] = temp2+4;
        T_ir[temp1+13] = temp2+5;
        T_ir[temp1+14] = temp2+6;
        
        /* Set T_jc */
        temp2 = 6*p;
        T_jc[temp2] = temp1;
        T_jc[temp2+1] = temp1+1;
        T_jc[temp2+2] = temp1+2;
        T_jc[temp2+3] = temp1+3;
        T_jc[temp2+4] = temp1+7;
        T_jc[temp2+5] = temp1+11;
        
        /* Calculate connection points */
        temp1 = 7*p+3;
        temp2 = 3*p;
        qx = q[temp1+1]*base/2.;
        qq = q[temp1+1]*q[temp1+1] + q[temp1+2]*q[temp1+2] + q[temp1+3]*q[temp1+3];
        pEdgeNext[temp2] = 2*q[temp1+1]*qx + q[temp1]*q[temp1]*base/2. - base/2*qq;
        pEdgeNext[temp2+1] = 2*q[temp1+2]*qx + q[temp1]*q[temp1+3]*base;
        pEdgeNext[temp2+2] = 2*q[temp1+3]*qx - q[temp1]*q[temp1+2]*base;
        
        qx = -qx;
        pEdgePrev[temp2] = 2*q[temp1+1]*qx - q[temp1]*q[temp1]*base/2. + base/2.*qq;
        pEdgePrev[temp2+1] = 2*q[temp1+2]*qx - q[temp1]*q[temp1+3]*base;
        pEdgePrev[temp2+2] = 2*q[temp1+3]*qx + q[temp1]*q[temp1+2]*base;
        
        /* Set local axis */
        qx = q[temp1+1];
        Ui[temp2] = 2*q[temp1+1]*qx + q[temp1]*q[temp1] - qq;
        Ui[temp2+1] = 2*q[temp1+2]*qx + 2*q[temp1]*q[temp1+3];
        Ui[temp2+2] = 2*q[temp1+3]*qx - 2*q[temp1]*q[temp1+2];
        
        qx = q[temp1+2];
        Uj[temp2] = 2*q[temp1+1]*qx - 2*q[temp1]*q[temp1+3];
        Uj[temp2+1] = 2*q[temp1+2]*qx + q[temp1]*q[temp1] - qq;
        Uj[temp2+2] = 2*q[temp1+3]*qx + 2*q[temp1]*q[temp1+1];
        
        qx = q[temp1+3];
        Uk[temp2] = 2*q[temp1+1]*qx + 2*q[temp1]*q[temp1+2];
        Uk[temp2+1] = 2*q[temp1+2]*qx - 2*q[temp1]*q[temp1+1];
        Uk[temp2+2] = 2*q[temp1+3]*qx + q[temp1]*q[temp1] - qq;
        
        /* Calculate new charge positions */
        qx = q[temp1+1]*red_body[temp2] + q[temp1+2]*red_body[temp2+1] + q[temp1+3]*red_body[temp2+2];
        tmpq[0] = q[temp1+2]*red_body[temp2+2] - q[temp1+3]*red_body[temp2+1];
        tmpq[1] = q[temp1+3]*red_body[temp2] - q[temp1+1]*red_body[temp2+2];
        tmpq[2] = q[temp1+1]*red_body[temp2+1] - q[temp1+2]*red_body[temp2];
        
        red[temp2] = 2*q[temp1+1]*qx + q[temp1]*q[temp1]*red_body[temp2] + 2*q[temp1]*tmpq[0] - red_body[temp2]*qq;
        red[temp2+1] = 2*q[temp1+2]*qx + q[temp1]*q[temp1]*red_body[temp2+1] + 2*q[temp1]*tmpq[1] - red_body[temp2+1]*qq;
        red[temp2+2] = 2*q[temp1+3]*qx + q[temp1]*q[temp1]*red_body[temp2+2] + 2*q[temp1]*tmpq[2] - red_body[temp2+2]*qq;
        
        qx = q[temp1+1]*blue_body[temp2] + q[temp1+2]*blue_body[temp2+1] + q[temp1+3]*blue_body[temp2+2];
        tmpq[0] = q[temp1+2]*blue_body[temp2+2] - q[temp1+3]*blue_body[temp2+1];
        tmpq[1] = q[temp1+3]*blue_body[temp2] - q[temp1+1]*blue_body[temp2+2];
        tmpq[2] = q[temp1+1]*blue_body[temp2+1] - q[temp1+2]*blue_body[temp2];
        
        blue[temp2] = 2*q[temp1+1]*qx + q[temp1]*q[temp1]*blue_body[temp2] + 2*q[temp1]*tmpq[0] - blue_body[temp2]*qq;
        blue[temp2+1] = 2*q[temp1+2]*qx + q[temp1]*q[temp1]*blue_body[temp2+1] + 2*q[temp1]*tmpq[1] - blue_body[temp2+1]*qq;
        blue[temp2+2] = 2*q[temp1+3]*qx + q[temp1]*q[temp1]*blue_body[temp2+2] + 2*q[temp1]*tmpq[2] - blue_body[temp2+2]*qq;
        
    } /* End of main loop */
    /* Set last elements of JC */
    M_jc[6*NP] = 12*NP;
    M_inv_jc[6*NP] = 12*NP;
    M_dot_jc[6*NP] = 9*NP;
    T_jc[6*NP] = 15*NP;
}