function mmolecules3(make_movie, movie_name)
% Simulates macromolecules
% Boolean make_movie: True to record movie
% String movie_name: Save movie to this file, must be provided if
%       make_movie == true
make_movie = true;
draw_charge = true;

% Set constant data
NP = 30; % Number of blocks
NC = 6*(NP-1)+6; % Number of constraints
NCPP = 6; % Number of constraints per particle
NT =6000; % Number of timesteps
dt = 0.05; % Timestep
nsteps = 5; % Update graphics every nsteps timestep
cstep = 1; % Steps since last graphic update
base = 2; % Base length of blocks
width = 0.5; % Width of blocks
height = 0.5; % Height of blocks
mass = 0.01; % Mass of blocks
jAngle = 2*pi/3.2; % Equilibrium angle around j axis
k_v = 0.001; % Velocity friction coefficient
k_w = 0.001; % Rotation friction coefficient
k_lj = 1e-10; % Lennard - Jones coefficient
s = 0; % Eq. distance of LJ, set automatically.

% Initialize matrices
q = zeros(7*NP,1); % Generalized position
v = zeros(6*NP,1); % Generalized velocity
M = sparse([],[],[],6*NP,6*NP,(3+3*3)*NP); % Generalized mass matrix
M_inv = sparse([],[],[],6*NP,6*NP,(3+3*3)*NP); % Inverse of M
M_dot = sparse([],[],[],6*NP,6*NP,(3+3*3)*NP); % dM/dt
I_body = sparse([],[],[],3*NP,3*NP,9*NP); % Body inertia matrix
I_body_inv = sparse([],[],[],3*NP,3*NP,9*NP); % Inverse of I_body
T = sparse([],[],[],7*NP,6*NP,(3+3*4)*NP); % dq/dt == T*v
F_ext = zeros(6*NP,1); % Generalized external force
phi = zeros(NC,1); % Constraints
G = sparse([],[],[],NC,6*NP,27*2*NP); % Jacobian
epsilon = sparse([],[],[],NC,NC,NC); % Constraint stiffness
pEdgeNext = zeros(3*NP,1); % Connection point to next block
pEdgePrev = zeros(3*NP,1); % Connection point to previous block
Ui = zeros(3*NP,1); % Local i axis
Uj = zeros(3*NP,1); % Local j axis
Uk = zeros(3*NP,1); % Local k axis
eqAngi = zeros(NP-1,1); % Equilibrium angle between i axis
eqAngj = zeros(NP-1,1); % Equilibrium angle between j axis
eqAngk = zeros(NP-1,1); % Equilibrium angle between k axis
A = zeros(1,3);
B = zeros(1,3);
C = zeros(1,3);
k_f = zeros(6*NP,1); % Friction coefficient
red = zeros(3*NP,1); % Red charges
red_body = zeros(3*NP,1); % Red charges (body coordinates)
blue = zeros(3*NP,1); % Blue charges
blue_body = zeros(3*NP,1); % Blue charges (body coordinates)
handle = zeros(1,NP);

if draw_charge
    handleB = zeros(1,NP);
    handleR = zeros(1,NP);
end

% Open movie file
if (make_movie)
    mov = avifile(movie_name);
    mov.Quality = 50;  
    mov.fps = 30;    
end

% Set friction coefficient
for p = 1:NP
    k_f(6*(p-1)+1:6*(p-1)+3) = k_v;
    k_f(6*(p-1)+4:6*(p-1)+6) = k_w;
end

% Open a new figure object
fp = figure;

% Set initial rotation of blocks
q(7*(NP-1)+4:7*(NP-1)+7) = [cos(0);sin(0)*[1;0;0]];
for p = (NP-1):-1:1
    q(7*(p-1)+4:7*(p-1)+7) = q(7*p+4:7*p+7); % Set to previous rotation
    
    % Rotate2 current block jAngle around j
    r = [cos(jAngle/2);sin(jAngle/2)*[0;1;0]];
    xc = 7*(p-1);
    q(7*(p-1)+4:7*(p-1)+7) = [r(1)*q(xc+4) - dot(r(2:4), q(xc+5:xc+7)); ...
        r(1)*q(xc+5:xc+7) + q(xc+4)*r(2:4) + cr(r(2:4), q(xc+5:xc+7))];
    
    % Rotate current block 2pi/32 (for example) around k
    r = [cos(pi/32);sin(pi/32)*[0;0;1]];
    q(7*(p-1)+4:7*(p-1)+7) = [r(1)*q(xc+4) - dot(r(2:4), q(xc+5:xc+7)); ...
        r(1)*q(xc+5:xc+7) + q(xc+4)*r(2:4) + cr(r(2:4), q(xc+5:xc+7))];
end

% Set initial block parameters
for p = 1:NP
    
    % Calculate body inertia matrix
    I_body(3*(p-1)+1:3*(p-1)+3, 3*(p-1)+1:3*(p-1)+3) = ...
        1/12*mass(1)*[width^2+height^2,0,0;0,base^2+ ...
        height^2,0;0,0,base^2+width^2];
    I_body_inv(3*(p-1)+1:3*(p-1)+3, 3*(p-1)+1:3*(p-1)+3) = ...
        inv(I_body(3*(p-1)+1:3*(p-1)+3, 3*(p-1)+1:3*(p-1)+3));
    
    % Get Rotation matrix
    R = getRot2(q(7*(p-1)+4:7*(p-1)+7));
    
    % Calculate temporary inertia matrix
    I = R*I_body(3*(p-1)+1:3*(p-1)+3,3*(p-1)+1:3*(p-1)+3)*R';
    I_inv = R*I_body_inv(3*(p-1)+1:3*(p-1)+3,3*(p-1)+1:3*(p-1)+3)*R';
    
    % Set mass matrix
    xc = 6*(p-1);
    M(xc+1:xc+3,xc+1:xc+3) = mass*eye(3);
    M(xc+4:xc+6,xc+4:xc+6) = I;
    M_inv(xc+1:xc+3,xc+1:xc+3) = 1/mass*eye(3);
    M_inv(xc+4:xc+6,xc+4:xc+6) = I_inv;
    omegaSTAR = getSTAR2(v(xc+4:xc+6));
    M_dot(xc+4:xc+6,xc+4:xc+6) = omegaSTAR*I + I*omegaSTAR;
    
    % Calculate T
    xc = 7*(p-1)+4;
    T(7*(p-1)+1:7*(p-1)+3,6*(p-1)+1:6*(p-1)+3) = eye(3);
    T(7*(p-1)+4:7*(p-1)+7,6*(p-1)+4:6*(p-1)+6) = ...
        [-q(xc+1),-q(xc+2),-q(xc+3);q(xc),q(xc+3),-q(xc+2); ...
        -q(xc+3),q(xc),q(xc+1);q(xc+2),-q(xc+1),q(xc)]*0.5;
    
    % Set connection points
    pEdgeNext(3*(p-1)+1:3*(p-1)+3) = Rotate2([base/2;0;0],q(7*(p-1)+4:7*(p-1)+7));
    pEdgePrev(3*(p-1)+1:3*(p-1)+3) = Rotate2([-base/2;0;0],q(7*(p-1)+4:7*(p-1)+7));
    
    % Set local axes
    Ui(3*(p-1)+1:3*(p-1)+3) = Rotate2([1;0;0],q(7*(p-1)+4:7*(p-1)+7));
    Uj(3*(p-1)+1:3*(p-1)+3) = Rotate2([0;1;0],q(7*(p-1)+4:7*(p-1)+7));
    Uk(3*(p-1)+1:3*(p-1)+3) = Rotate2([0;0;1],q(7*(p-1)+4:7*(p-1)+7));
    
    % Visualize using patch
    vertex_matrix(:,:,p)=[0,0,0;1,0,0;1,1,0;0,1,0;0,0,1;1,0,1;1,1,1;0,1,1]-0.5;
    vertex_matrix(:,1,p)=vertex_matrix(:,1,p)*base;
    vertex_matrix(:,2,p)=vertex_matrix(:,2,p)*width;
    vertex_matrix(:,3,p)=vertex_matrix(:,3,p)*height;
    faces_matrix=[1,2,6,5;2,3,7,6;3,4,8,7;4,1,5,8;1,2,3,4;5,6,7,8];
    
    % Rotate vertex matrix
    clear v_m;
    for i = 1:8
        v_m(i,:) = Rotate2(vertex_matrix(i,:,p),q(7*(p-1)+4:7*(p-1)+7))';
    end
    xc = 7*(p-1);
    v_m=[v_m(:,1)+q(xc+1),v_m(:,2)+q(xc+2),v_m(:,3)+q(xc+3)];
    handle(p) = patch('Vertices', v_m, 'Faces', faces_matrix, ...
        'FaceVertexCData', hsv(8), 'FaceColor', 'interp');
end
ax = gca;
% view([77 4]); % Set point of view
view([90 90]); % Set point of view
axis(ax, [-5 15 -5 15 -5 15]);
daspect([1 1 1]);

% Position the blocks at correct edgepoints
p = NP-1;
% Set position of last block
q(7*(NP-1)+1:7*(NP-1)+3) = [-base(1)/2 0 0]';

while p >= 1
    % Calculate distance between edgepoints
    d = q(7*p+1:7*p+3)+pEdgePrev(3*p+1:3*p+3) - ...
        q(7*(p-1)+1:7*(p-1)+3)-pEdgeNext(3*(p-1)+1:3*(p-1)+3);
    q(7*(p-1)+1:7*(p-1)+3) = q(7*(p-1)+1:7*(p-1)+3) + d;
    p = p-1;
end

% Calculate equilibrium angles
for p = 1:(NP-1)
    di_dot = dot(Ui(3*(p-1)+1:3*(p-1)+3), Ui(3*p+1:3*p+3));
    dj_dot = dot(Uj(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
    dk_dot = dot(Uk(3*(p-1)+1:3*(p-1)+3), Uk(3*p+1:3*p+3));
    di_x = cr(Ui(3*(p-1)+1:3*(p-1)+3), Ui(3*p+1:3*p+3));
    dj_x = cr(Uj(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
    dk_x = cr(Uk(3*(p-1)+1:3*(p-1)+3), Uk(3*p+1:3*p+3));
    
    % Calculate angles
    eqAngi(p) = atan2(norm(di_x),di_dot);
    eqAngj(p) = atan2(norm(dj_x),dj_dot);
    eqAngk(p) = atan2(norm(dk_x),dk_dot);
end

% Place red charges
for p = 1:NP
    red_body(3*(p-1)+1:3*(p-1)+3) = [0;width/2;0];
    red(3*(p-1)+1:3*(p-1)+3) = Rotate2(red_body(3*(p-1)+1:3*(p-1)+3), ...
        q(7*(p-1)+4:7*(p-1)+7)) + q(7*(p-1)+1:7*(p-1)+3);
end

% Place blue charges
for p = 1:(NP-3)
    % Calculate p+3 red in coordinate frame p
    redp = red(3*(p+2)+1:3*(p+2)+3) - q(7*(p-1)+1:7*(p-1)+3);
    redp = Rotate2(redp, [q(7*(p-1)+4); -1*q(7*(p-1)+5:7*(p-1)+7)]);
    
    % Calculate r_h in frame p
    r_h = Uj(3*(p+2)+1:3*(p+2)+3);
    r_h = Rotate2(r_h, [q(7*(p-1)+4); -1*q(7*(p-1)+5:7*(p-1)+7)]);
    
    % Calculate r_abs
    r_abs = -(width/2 + redp(2))/r_h(2);
    
    % Calculate blue
    blue_body(3*(p-1)+1:3*(p-1)+3) = redp + r_abs*r_h;
    blue(3*(p-1)+1:3*(p-1)+3) = ...
        Rotate2(blue_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
        q(7*(p-1)+1:7*(p-1)+3);
end

% Place last blue charges
for p = (NP-2):NP
    blue_body(3*(p-1)+1:3*(p-1)+3) = [0;-width/2;0];
    blue(3*(p-1)+1:3*(p-1)+3) = Rotate2(blue_body(3*(p-1)+1:3*(p-1)+3), ...
        q(7*(p-1)+4:7*(p-1)+7)) + q(7*(p-1)+1:7*(p-1)+3);
end

% Calculate equilibrium distance of LJ potential
s = (norm(red(1:3) - blue(3*3+1:3*3+3)))/2^(1/6);


hold on;

% Draw spheres at chargepoints
if draw_charge
	[x,y,z] = sphere;
	x = x*0.05;
	y = y*0.05;
	z = z*0.05;
	for p = 1:NP
        handleB(p) = surf(x+blue(3*(p-1)+1), y+blue(3*(p-1)+2), z+blue(3*(p-1)+3));
        handleR(p) = surf(x+red(3*(p-1)+1), y+red(3*(p-1)+2), z+red(3*(p-1)+3));
        set(handleB(p), 'EdgeAlpha', 0);
        set(handleR(p), 'EdgeAlpha', 0);
        set(handleB(p), 'FaceColor', [0 0 1]);
        set(handleR(p), 'FaceColor', [1 0 0]);
	end
end

% Update graphics
for p = 1:NP
    clear v_m;
    for i = 1:8
        v_m(i,:) = Rotate2(vertex_matrix(i,:,p),q(7*(p-1)+4:7*(p-1)+7))';
    end
    xc = 7*(p-1);
    v_m=[v_m(:,1)+q(xc+1),v_m(:,2)+q(xc+2),v_m(:,3)+q(xc+3)];
    set(handle(p), 'Vertices', v_m);
end

% Start main loop
for t = 1:NT
    % Set matrices to zeros
    G = G*0;
    phi = phi*0;
    F_ext = F_ext*0;
    
    % Calculate phi and G for each block
    for p = 1:NP
        qI = 7*(p-1);
        vI = 6*(p-1);
        
        if (p ~= NP)
            % Calculate angles
            di_dot = Ui(3*(p-1)+1:3*(p-1)+3)'*Ui(3*p+1:3*p+3);
            dj_dot = Uj(3*(p-1)+1:3*(p-1)+3)'*Uj(3*p+1:3*p+3);
            dk_dot = Uk(3*(p-1)+1:3*(p-1)+3)'*Uk(3*p+1:3*p+3);
            di_x = cr(Ui(3*(p-1)+1:3*(p-1)+3), Ui(3*p+1:3*p+3));
            dj_x = cr(Uj(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
            dk_x = cr(Uk(3*(p-1)+1:3*(p-1)+3), Uk(3*p+1:3*p+3));
            
            theta1 = atan2(norm(di_x), di_dot);
            theta2 = atan2(norm(dk_x), dk_dot);
            theta3 = atan2(norm(dj_x), dj_dot);
            
            % Calculate phi
            phi(NCPP*(p-1)+1:NCPP*(p-1)+3) = q(qI+1:qI+3) + pEdgeNext(3*(p-1)+1:3*(p-1)+3) - ...
                q(7*p+1:7*p+3) - pEdgePrev(3*p+1:3*p+3);
            phi(NCPP*(p-1)+4) = theta1 - eqAngi(p);
            phi(NCPP*(p-1)+5) = theta2 - eqAngk(p);
            phi(NCPP*(p-1)+6) = theta3 - eqAngj(p);
            
            % Calculate Jacobian
            edgeNextSTAR = getSTAR2(pEdgeNext(3*(p-1)+1:3*(p-1)+3));
            edgePrevSTAR = getSTAR2(pEdgePrev(3*p+1:3*p+3));
            A = -1/(di_x'*di_x + di_dot^2)*(norm(di_x) + di_dot^2/(norm(di_x)+eps))*di_x;
            B = -1/(dk_x'*dk_x + dk_dot^2)*(norm(dk_x) + dk_dot^2/(norm(dk_x)+eps))*dk_x;
            C = -1/(dj_x'*dj_x + dj_dot^2)*(norm(dj_x) + dj_dot^2/(norm(dj_x)+eps))*dj_x;
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, NCPP*(p-1)+1:NCPP*(p-1)+3) = eye(3);
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, NCPP*(p-1)+4:NCPP*(p-1)+6) = -edgeNextSTAR;
            G(NCPP*(p-1)+4, NCPP*(p-1)+4:NCPP*(p-1)+6) = A(:)';
            G(NCPP*(p-1)+5, NCPP*(p-1)+4:NCPP*(p-1)+6) = B(:)';
            G(NCPP*(p-1)+6, NCPP*(p-1)+4:NCPP*(p-1)+6) = C(:)';
            
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, NCPP*p+1:NCPP*p+3) = -eye(3);
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, NCPP*p+4:NCPP*p+6) = edgePrevSTAR;
            G(NCPP*(p-1)+4, NCPP*p+4:NCPP*p+6) = -A(:)';
            G(NCPP*(p-1)+5, NCPP*p+4:NCPP*p+6) = -B(:)';
            G(NCPP*(p-1)+6, NCPP*p+4:NCPP*p+6) = -C(:)';
        else
            % Lock last block in origo
            % Set phi
            phi(NCPP*(p-1)+1:NCPP*(p-1)+3) = q(qI+1:qI+3) + pEdgeNext(3*(p-1)+1:3*(p-1)+3);
            phi(NCPP*(p-1)+4) = Ui(3*(p-1)+1:3*(p-1)+3)'*[0;0;1];
            phi(NCPP*(p-1)+5) = Ui(3*(p-1)+1:3*(p-1)+3)'*[0;1;0];
            phi(NCPP*(p-1)+6) = Uj(3*(p-1)+1:3*(p-1)+3)'*[0;0;1];
            A(1:3) = cr(Ui(3*(p-1)+1:3*(p-1)+3), [0;0;1]);
            B(1:3) = cr(Ui(3*(p-1)+1:3*(p-1)+3), [0;1;0]);
            C(1:3) = cr(Uj(3*(p-1)+1:3*(p-1)+3), [0;0;1]);
            
            % Set Jacobian
            edgeNextSTAR = getSTAR2(pEdgeNext(3*(p-1)+1:3*(p-1)+3));
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, 6*(p-1)+1:6*(p-1)+3) = eye(3);
            G(NCPP*(p-1)+1:NCPP*(p-1)+3, 6*(p-1)+4:6*(p-1)+6) = -edgeNextSTAR;
            G(NCPP*(p-1)+4, 6*(p-1)+4:6*(p-1)+6) = A(:)';
            G(NCPP*(p-1)+5, 6*(p-1)+4:6*(p-1)+6) = B(:)';
            G(NCPP*(p-1)+6, 6*(p-1)+4:6*(p-1)+6) = C(:)';
        end
        
%         % Add Lennard - Jones Force
%         for p2 = (p+1):(NP-1)
%             % Calculate force red(p) <- blue(p2)
%             r_h = red(3*(p-1)+1:3*(p-1)+3) - blue(3*(p2-1)+1:3*(p2-1)+3);
%             r = norm(r_h);
%             r_h = r_h/r;
%             F = 4*k_lj*(12*s^12/r^13 - 6*s^6/r^7)*r_h;
%             F_ext(6*(p-1)+1:6*(p-1)+3) = F_ext(6*(p-1)+1:6*(p-1)+3) + F;
%             F_ext(6*(p2-1)+1:6*(p2-1)+3) = F_ext(6*(p2-1)+1:6*(p2-1)+3) - F;
%             F_ext(6*(p-1)+4:6*(p-1)+6) = F_ext(6*(p-1)+4:6*(p-1)+6) + ...
%                 cr(red(3*(p-1)+1:3*(p-1)+3), F);
%             F_ext(6*(p2-1)+4:6*(p2-1)+6) = F_ext(6*(p2-1)+4:6*(p2-1)+6) + ...
%                 cr(blue(3*(p2-1)+1:3*(p2-1)+3), -1*F);
%             
%             % Calculate force red(p2) <- blue(p)
%             r_h = red(3*(p2-1)+1:3*(p2-1)+3) - blue(3*(p2-1)+1:3*(p2-1)+3);
%             r = norm(r_h);
%             r_h = r_h/r;
%             F = 4*k_lj*(12*s^12/r^13 - 6*s^6/r^7)*r_h;
%             F_ext(6*(p2-1)+1:6*(p2-1)+3) = F_ext(6*(p2-1)+1:6*(p2-1)+3) + F;
%             F_ext(6*(p-1)+1:6*(p-1)+3) = F_ext(6*(p-1)+1:6*(p-1)+3) - F;
%             F_ext(6*(p2-1)+4:6*(p2-1)+6) = F_ext(6*(p2-1)+4:6*(p2-1)+6) + ...
%                 cr(red(3*(p2-1)+1:3*(p2-1)+3), F);
%             F_ext(6*(p-1)+4:6*(p-1)+6) = F_ext(6*(p-1)+4:6*(p-1)+6) + ...
%                 cr(blue(3*(p-1)+1:3*(p-1)+3), -1*F);
%         end
    end

    % Set external forces
    if t < 2000
        F_ext(2) = F_ext(2) + 0.00001*t;
    elseif t < 4000
        F_ext(2) = F_ext(2) + 0.00001*(4000-t);
    end
    if t == 1000
        t
    end
    
%   Add Friction
    F_ext = F_ext - k_f.*v;
    
    % Add motion
    F_ext = F_ext - M_dot*v;
    
    % Set epsilon
    epsilon = 0*epsilon;
    EPS0 = 0;
    EPS1 = 0.01;
    EPS2 = 0.1;
    EPS3 = 0.1;
    for i = 1:(NP-1)
        epsilon(NCPP*(i-1)+1:NCPP*(i-1)+3, NCPP*(i-1)+1:NCPP*(i-1)+3) = eye(3)*EPS0;
        epsilon(NCPP*(i-1)+4, NCPP*(i-1)+4) = EPS1;
        epsilon(NCPP*(i-1)+5, NCPP*(i-1)+5) = EPS2;
        epsilon(NCPP*(i-1)+6, NCPP*(i-1)+6) = EPS3;
    end
    epsilon(NCPP*(NP-1)+1:NCPP*(NP-1)+3, NCPP*(NP-1)+1:NCPP*(NP-1)+3) = eye(3)*0;
    epsilon(NCPP*(NP-1)+4, NCPP*(NP-1)+4) = 0;
    epsilon(NCPP*(NP-1)+5, NCPP*(NP-1)+5) = 0;
    epsilon(NCPP*(NP-1)+6, NCPP*(NP-1)+6) = 0;
    
    % Solve equation of motion
    % Define Schur complement S
    S = G*M_inv*G' + dt^-2*epsilon;
    
    % Calculate lambda
    lambda = S\(-G*v - dt*G*M_inv*F_ext - dt^-1*phi)/dt;
    
    % Compute velocity
    v = M\(G'*lambda*dt + M*v + dt*F_ext);
    
    % Update positions
    q = q + dt*T*v;
%     An = [M, -dt*G'; G, dt^-1*epsilon];
%     b = [M*v+dt*F_ext; -dt^-1*phi];
%     u = An\b;
%     v = u(1:6*NP);
%     q = q + dt*T*v;
    
    % Update all other body properties
    % Set matrices to zero
    M_dot = M_dot*0;

    for p = 1:NP
        qI = 7*(p-1);
        vI = 6*(p-1);
        
        % Get rotation matrix
        R = getRot2(q(qI+4:qI+7));
        
        % Set temporary inertia matrix
        I = R*I_body(3*(p-1)+1:3*(p-1)+3,3*(p-1)+1:3*(p-1)+3)*R';
        I_inv = R*I_body_inv(3*(p-1)+1:3*(p-1)+3,3*(p-1)+1:3*(p-1)+3)*R';
        
        % Set mass matrix
        xc = 6*(p-1);
        M(xc+4:xc+6, xc+4:xc+6) = I;
        M_inv(xc+4:xc+6, xc+4:xc+6) = I_inv;
        omegaSTAR = getSTAR2(v(vI+4:vI+6));
        M_dot(xc+4:xc+6, xc+4:xc+6) = omegaSTAR*I + I*omegaSTAR;
        
        % Calculate T
        xc = 7*(p-1)+4;
        T(7*(p-1)+4:7*(p-1)+7, 6*(p-1)+4:6*(p-1)+6) = ...
            [-q(xc+1), -q(xc+2), -q(xc+3); q(xc), q(xc+3), -q(xc+2); ...
            -q(xc+3), q(xc), q(xc+1); q(xc+2), -q(xc+1), q(xc)]*0.5;
        
        % Set connection points
        pEdgeNext(3*(p-1)+1:3*(p-1)+3) = Rotate2([base/2;0;0], q(qI+4:qI+7));
        pEdgePrev(3*(p-1)+1:3*(p-1)+3) = Rotate2([-base/2;0;0], q(qI+4:qI+7));
        
        % Set local axes
        Ui(3*(p-1)+1:3*(p-1)+3) = Rotate2([1;0;0], q(qI+4:qI+7));
        Uj(3*(p-1)+1:3*(p-1)+3) = Rotate2([0;1;0], q(qI+4:qI+7));
        Uk(3*(p-1)+1:3*(p-1)+3) = Rotate2([0;0;1], q(qI+4:qI+7));
        
        % Calculate new chargepositions
        red(3*(p-1)+1:3*(p-1)+3) = Rotate2(red_body(3*(p-1)+1:3*(p-1)+3), ...
            q(qI+4:qI+7)) + q(qI+1:qI+3);
        blue(3*(p-1)+1:3*(p-1)+3) = Rotate2(blue_body(3*(p-1)+1:3*(p-1)+3), ...
            q(qI+4:qI+7)) + q(qI+1:qI+3);
    end
    
    % Update Graphics
    if cstep == nsteps
        for p = 1:NP
            qI = 7*(p-1);
            vI = 6*(p-1);
            
            % Normalize quaternion
            qNorm = sqrt(q(qI+4)*q(qI+4) + dot(q(qI+5:qI+7),q(qI+5:qI+7)));
            q(qI+4:qI+7) = q(qI+4:qI+7)/qNorm;
            
            % Rotate vertex matrix
            clear v_m;
            for i = 1:8
                v_m(i,:) = Rotate2(vertex_matrix(i,:,p), q(qI+4:qI+7))';
            end
            v_m=[v_m(:,1)+q(qI+1),v_m(:,2)+q(qI+2),v_m(:,3)+q(qI+3)];
            set(handle(p),'Vertices',v_m);
            
            % Set charge positions
            if draw_charge
                set(handleB(p), 'XData', x+blue(3*(p-1)+1));
                set(handleB(p), 'YData', y+blue(3*(p-1)+2));
                set(handleB(p), 'ZData', z+blue(3*(p-1)+3));
                set(handleR(p), 'XData', x+red(3*(p-1)+1));
                set(handleR(p), 'YData', y+red(3*(p-1)+2));
                set(handleR(p), 'ZData', z+red(3*(p-1)+3));
            end
        end
        drawnow;
        cstep = 1;
        
        % Record movie
        if (make_movie)
            frame = getframe;
            mov = addframe(mov, frame);
        end
    else
        cstep = cstep+1;
    end
end

if (make_movie)
    mov = close(mov);
end