#include "matrix.h"

void rotate(double *z, const double *x, const double *q)
{
    double qx;
    double qq;
    double tmp2[3];
    
    qx = q[1]*x[0] + q[2]*x[1] + q[3]*x[2];
    qq = q[1]*q[1] + q[2]*q[2] + q[3]*q[3];
    tmp2[0] = q[2]*x[2] - q[3]*x[1];
    tmp2[1] = q[3]*x[0] - q[1]*x[2];
    tmp2[2] = q[1]*x[1] - q[2]*x[0];
    
    z[0] = 2*q[1]*qx + q[0]*q[0]*x[0] + 2*q[0]*tmp2[0] - x[0]*qq;
    z[1] = 2*q[2]*qx + q[0]*q[0]*x[1] + 2*q[0]*tmp2[1] - x[1]*qq;
    z[2] = 2*q[3]*qx + q[0]*q[0]*x[2] + 2*q[0]*tmp2[2] - x[2]*qq;
}

/*
 * [M, M_inv, M_dot, T, pEdgeNext, pEdgePrev, Ui, Uj, Uk, red, blue] =
 *   updateBlocks(NP, q, v, base, I, I_inv, red_body, blue_body, m);
 * Updates block properties from the provided parameters
 *
 * Parameters:
 * NP: Number of blocks
 * q: Generalized position of blocks
 * v: Generalized velocity of blocks
 * base: Length of blocks
 * I: The general inertia tensor
 * I_inv: General inverse inertia tensor
 * red_body: Positions of red charges in body coordinates
 * blue_body: Position of blue charges in body coordinates
 * m: Mass of blocks
 *
 * Returns:
 * M: Mass matrix
 * M_inv: Inverse mass matrix
 * M_dot: Time derivative of mass matrix
 * T: 
 * pEdgeNext: Position of forward edgepoint
 * pEdgePrev: Position of backward edgepoint
 * Ui, Uj, Uk: Local body axes
 * red: Position of red charges
 * blue: Position of blue charges
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Input arguments */
    int NP;
    const double *q;
    const double *v;
    double base;
    const double *I;
    const double *I_inv;
    const double *red_body;
    const double *blue_body;
    double m;
    /* Output arguments */
    double *M;
    int *M_ir;
    int *M_jc;
    double *M_inv;
    int *M_inv_ir;
    int *M_inv_jc;
    double *M_dot;
    int *M_dot_ir;
    int *M_dot_jc;
    double *T;
    int *T_ir;
    int *T_jc;
    double *pEdgeNext;
    double *pEdgePrev;
    double *Ui, *Uj, *Uk;
    double *red, *blue;
    /* Others */
    int p = 0;
    int tmp1;
    int tmp2;
    int col;
    int row;
    double m_inv;
    double omegaSTAR[9];
    double qx,qq;
    double tmpq[3];
    
    /* Check for correct number of arguments */
    if (nlhs != 11 || nrhs != 9)
        mexErrMsgTxt("Wrong number of arguments");
    
    /* Get pointers to input */
    NP = *mxGetPr(prhs[0]);
    q = mxGetPr(prhs[1]);
    v = mxGetPr(prhs[2]);
    base = *mxGetPr(prhs[3]);
    I = mxGetPr(prhs[4]);
    I_inv = mxGetPr(prhs[5]);
    red_body = mxGetPr(prhs[6]);
    blue_body = mxGetPr(prhs[7]);
    m = *mxGetPr(prhs[8]);
    
    /* Reserve memory */
    plhs[0] = mxCreateSparse(6*NP, 6*NP, 12*NP, mxREAL); /* M */
    plhs[1] = mxCreateSparse(6*NP, 6*NP, 12*NP, mxREAL); /* M_inv */
    plhs[2] = mxCreateSparse(6*NP, 6*NP, 9*NP, mxREAL); /* M_dot */
    plhs[3] = mxCreateSparse(7*NP, 6*NP, 15*NP, mxREAL); /* T */
    plhs[4] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* pEdgeNext */
    plhs[5] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* pEdgePrev */
    plhs[6] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* Ui */
    plhs[7] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* Uj */
    plhs[8] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* Uk */
    plhs[9] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* red */
    plhs[10] = mxCreateDoubleMatrix(3*NP,1,mxREAL); /* blue */
    
    /* Get pointer to output */
    M = mxGetPr(plhs[0]);
    M_ir = mxGetIr(plhs[0]);
    M_jc = mxGetJc(plhs[0]);
    M_inv = mxGetPr(plhs[1]);
    M_inv_ir = mxGetIr(plhs[1]);
    M_inv_jc = mxGetJc(plhs[1]);
    M_dot = mxGetPr(plhs[2]);
    M_dot_ir = mxGetIr(plhs[2]);
    M_dot_jc = mxGetJc(plhs[2]);
    T = mxGetPr(plhs[3]);
    T_ir = mxGetIr(plhs[3]);
    T_jc = mxGetJc(plhs[3]);
    pEdgeNext = mxGetPr(plhs[4]);
    pEdgePrev = mxGetPr(plhs[5]);
    Ui = mxGetPr(plhs[6]);
    Uj = mxGetPr(plhs[7]);
    Uk = mxGetPr(plhs[8]);
    red = mxGetPr(plhs[9]);
    blue = mxGetPr(plhs[10]);
    
    m_inv = 1/m;
    /* Main loop */
    for (p = 0; p < NP; p++)
    {
        /* Create T */
        tmp1 = 15*p;
        tmp2 = 7*p+3;
        T[tmp1] = 1;
        T[tmp1+1] = 1;
        T[tmp1+2] = 1;
        
        T[tmp1+3] = -q[tmp2+1]/2.;
        T[tmp1+4] = q[tmp2]/2.;
        T[tmp1+5] = -q[tmp2+3]/2.;
        T[tmp1+6] = q[tmp2+2]/2.;
        
        T[tmp1+7] = -q[tmp2+2]/2.;
        T[tmp1+8] = q[tmp2+3]/2.;
        T[tmp1+9] = q[tmp2]/2.;
        T[tmp1+10] = -q[tmp2+1]/2.;
        
        T[tmp1+11] = -q[tmp2+3]/2.;
        T[tmp1+12] = -q[tmp2+2]/2.;
        T[tmp1+13] = q[tmp2+1]/2.;
        T[tmp1+14] = q[tmp2]/2.;
        
        tmp2 = 7*p;
        T_ir[tmp1] = tmp2;
        T_ir[tmp1+1] = tmp2+1;
        T_ir[tmp1+2] = tmp2+2;
        T_ir[tmp1+3] = tmp2+3;
        T_ir[tmp1+4] = tmp2+4;
        T_ir[tmp1+5] = tmp2+5;
        T_ir[tmp1+6] = tmp2+6;
        T_ir[tmp1+7] = tmp2+3;
        T_ir[tmp1+8] = tmp2+4;
        T_ir[tmp1+9] = tmp2+5;
        T_ir[tmp1+10] = tmp2+6;
        T_ir[tmp1+11] = tmp2+3;
        T_ir[tmp1+12] = tmp2+4;
        T_ir[tmp1+13] = tmp2+5;
        T_ir[tmp1+14] = tmp2+6;
        
        tmp2 = 6*p;
        T_jc[tmp2] = tmp1;
        T_jc[tmp2+1] = tmp1+1;
        T_jc[tmp2+2] = tmp1+2;
        T_jc[tmp2+3] = tmp1+3;
        T_jc[tmp2+4] = tmp1+7;
        T_jc[tmp2+5] = tmp1+11;
        
        /* Calculate M */
        tmp1 = 12*p;
        tmp2 = 9*p;
        M[tmp1] = m;
        M[tmp1+1] = m;
        M[tmp1+2] = m;
        
        M[tmp1+3] = I[tmp2];
        M[tmp1+4] = I[tmp2+1];
        M[tmp1+5] = I[tmp2+2];
        
        M[tmp1+6] = I[tmp2+3];
        M[tmp1+7] = I[tmp2+4];
        M[tmp1+8] = I[tmp2+5];
        
        M[tmp1+9] = I[tmp2+6];
        M[tmp1+10] = I[tmp2+7];
        M[tmp1+11] = I[tmp2+8];
        
        tmp2 = 6*p;
        M_ir[tmp1] = tmp2;
        M_ir[tmp1+1] = tmp2+1;
        M_ir[tmp1+2] = tmp2+2;
        M_ir[tmp1+3] = tmp2+3;
        M_ir[tmp1+4] = tmp2+4;
        M_ir[tmp1+5] = tmp2+5;
        M_ir[tmp1+6] = tmp2+3;
        M_ir[tmp1+7] = tmp2+4;
        M_ir[tmp1+8] = tmp2+5;
        M_ir[tmp1+9] = tmp2+3;
        M_ir[tmp1+10] = tmp2+4;
        M_ir[tmp1+11] = tmp2+5;
        
        tmp2 = 6*p;
        M_jc[tmp2] = tmp1;
        M_jc[tmp2+1] = tmp1+1;
        M_jc[tmp2+2] = tmp1+2;
        M_jc[tmp2+3] = tmp1+3;
        M_jc[tmp2+4] = tmp1+6;
        M_jc[tmp2+5] = tmp1+9;
        
        /* Calculate M_inv */
        tmp1 = 12*p;
        tmp2 = 9*p;
        M_inv[tmp1] = m_inv;
        M_inv[tmp1+1] = m_inv;
        M_inv[tmp1+2] = m_inv;
        
        M_inv[tmp1+3] = I_inv[tmp2];
        M_inv[tmp1+4] = I_inv[tmp2+1];
        M_inv[tmp1+5] = I_inv[tmp2+2];
        
        M_inv[tmp1+6] = I_inv[tmp2+3];
        M_inv[tmp1+7] = I_inv[tmp2+4];
        M_inv[tmp1+8] = I_inv[tmp2+5];
        
        M_inv[tmp1+9] = I_inv[tmp2+6];
        M_inv[tmp1+10] = I_inv[tmp2+7];
        M_inv[tmp1+11] = I_inv[tmp2+8];
        
        tmp2 = 6*p;
        M_inv_ir[tmp1] = tmp2;
        M_inv_ir[tmp1+1] = tmp2+1;
        M_inv_ir[tmp1+2] = tmp2+2;
        
        M_inv_ir[tmp1+3] = tmp2+3;
        M_inv_ir[tmp1+4] = tmp2+4;
        M_inv_ir[tmp1+5] = tmp2+5;
        M_inv_ir[tmp1+6] = tmp2+3;
        M_inv_ir[tmp1+7] = tmp2+4;
        M_inv_ir[tmp1+8] = tmp2+5;
        M_inv_ir[tmp1+9] = tmp2+3;
        M_inv_ir[tmp1+10] = tmp2+4;
        M_inv_ir[tmp1+11] = tmp2+5;
        
        M_inv_jc[tmp2] = tmp1;
        M_inv_jc[tmp2+1] = tmp1+1;
        M_inv_jc[tmp2+2] = tmp1+2;
        M_inv_jc[tmp2+3] = tmp1+3;
        M_inv_jc[tmp2+4] = tmp1+6;
        M_inv_jc[tmp2+5] = tmp1+9;
        
        /* Calculate M_dot */
        tmp1 = 6*p+3;
        omegaSTAR[0] = 0;
        omegaSTAR[1] = v[tmp1+2];
        omegaSTAR[2] = -v[tmp1+1];
        omegaSTAR[3] = -v[tmp1+2];
        omegaSTAR[4] = 0;
        omegaSTAR[5] = v[tmp1];
        omegaSTAR[6] = v[tmp1+1];
        omegaSTAR[7] = -v[tmp1];
        omegaSTAR[8] = 0;
        
        tmp1 = 9*p;
        for (col = 0; col < 3; col++)
        {
            for (row = 0; row < 3; row++)
            {
                M_dot[tmp1+col*3+row] = omegaSTAR[row]*I[tmp1+col*3] + omegaSTAR[row+3]*I[tmp1+col*3+1] + omegaSTAR[row+6]*I[tmp1+col*3+2];
                M_dot[tmp1+col*3+row] = M_dot[tmp1+col*3+row] + omegaSTAR[col*3]*I[tmp1+row] + omegaSTAR[col*3+1]*I[tmp1+row+3] + omegaSTAR[col*3+2]*I[tmp1+row+6];
            }
        }
        tmp2 = 6*p;
        M_dot_ir[tmp1] = tmp2+3;
        M_dot_ir[tmp1+1] = tmp2+4;
        M_dot_ir[tmp1+2] = tmp2+5;
        M_dot_ir[tmp1+3] = tmp2+3;
        M_dot_ir[tmp1+4] = tmp2+4;
        M_dot_ir[tmp1+5] = tmp2+5;
        M_dot_ir[tmp1+6] = tmp2+3;
        M_dot_ir[tmp1+7] = tmp2+4;
        M_dot_ir[tmp1+8] = tmp2+5;
        
        if (p != 0)
            M_dot_jc[tmp2] = M_dot_jc[tmp2-1]+3;
        M_dot_jc[tmp2+1] = M_dot_jc[tmp2];
        M_dot_jc[tmp2+2] = M_dot_jc[tmp2+1];
        M_dot_jc[tmp2+3] = tmp1;
        M_dot_jc[tmp2+4] = tmp1+3;
        M_dot_jc[tmp2+5] = tmp1+6;
        
        /* Calculate connection points */
        tmp1 = 7*p+3;
        tmp2 = 3*p;
        qx = q[tmp1+1]*base/2.;
        qq = q[tmp1+1]*q[tmp1+1] + q[tmp1+2]*q[tmp1+2] + q[tmp1+3]*q[tmp1+3];
        pEdgeNext[tmp2] = 2*q[tmp1+1]*qx + q[tmp1]*q[tmp1]*base/2. - base/2*qq;
        pEdgeNext[tmp2+1] = 2*q[tmp1+2]*qx + q[tmp1]*q[tmp1+3]*base;
        pEdgeNext[tmp2+2] = 2*q[tmp1+3]*qx - q[tmp1]*q[tmp1+2]*base;
        
        qx = -qx;
        pEdgePrev[tmp2] = 2*q[tmp1+1]*qx - q[tmp1]*q[tmp1]*base/2. + base/2.*qq;
        pEdgePrev[tmp2+1] = 2*q[tmp1+2]*qx - q[tmp1]*q[tmp1+3]*base;
        pEdgePrev[tmp2+2] = 2*q[tmp1+3]*qx + q[tmp1]*q[tmp1+2]*base;
        
        /* Set local axis */
        qx = q[tmp1+1];
        Ui[tmp2] = 2*q[tmp1+1]*qx + q[tmp1]*q[tmp1] - qq;
        Ui[tmp2+1] = 2*q[tmp1+2]*qx + 2*q[tmp1]*q[tmp1+3];
        Ui[tmp2+2] = 2*q[tmp1+3]*qx - 2*q[tmp1]*q[tmp1+2];
        
        qx = q[tmp1+2];
        Uj[tmp2] = 2*q[tmp1+1]*qx - 2*q[tmp1]*q[tmp1+3];
        Uj[tmp2+1] = 2*q[tmp1+2]*qx + q[tmp1]*q[tmp1] - qq;
        Uj[tmp2+2] = 2*q[tmp1+3]*qx + 2*q[tmp1]*q[tmp1+1];
        
        qx = q[tmp1+3];
        Uk[tmp2] = 2*q[tmp1+1]*qx + 2*q[tmp1]*q[tmp1+2];
        Uk[tmp2+1] = 2*q[tmp1+2]*qx - 2*q[tmp1]*q[tmp1+1];
        Uk[tmp2+2] = 2*q[tmp1+3]*qx + q[tmp1]*q[tmp1] - qq;
        
        /* Calculate new charge positions */
        tmp2 = 3*p;
        tmp1 = 7*p;
        rotate(red+tmp2, red_body+tmp2, q+tmp1+3);
        rotate(blue+tmp2, blue_body+tmp2, q+tmp1+3);
        red[tmp2] = red[tmp2] + q[tmp1];
        red[tmp2+1] = red[tmp2+1] + q[tmp1+1];
        red[tmp2+2] = red[tmp2+2] + q[tmp1+2];
        blue[tmp2] = blue[tmp2] + q[tmp1];
        blue[tmp2+1] = blue[tmp2+1] + q[tmp1+1];
        blue[tmp2+2] = blue[tmp2+2] + q[tmp1+2];
    }
    M_jc[6*NP] = 12*NP;
    M_inv_jc[6*NP] = 12*NP;
    M_dot_jc[6*NP] = 9*NP;
    T_jc[6*NP] = 15*NP;
}
