#include "matrix.h"

/*
 * [I I_inv] = getInertia(NP, Ig, Ig_inv)
 * Takes the generalized inertia matrices and transform them to
 * three dimensional inertia matrices
 * 
 * Parameters:
 * NP: Number of particles
 * Ig: Generalized inertia matrix
 * Ig_inv: inverse generalized inertia matrix
 *
 * Returns:
 * I: All inertia matrices
 * I: All inverse inertia matrices
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Input arguments */
    int NP;
    const double *Ig;
    const int *Ig_ir;
    const int *Ig_jc;
    const double *Ig_inv;
    const int *Ig_inv_ir;
    const int *Ig_inv_jc;
    /* Output arguments */
    double *I;
    double *I_inv;
    /* Other variables */
    int dims[3];
    int p = 0;
    int i;
    
    /* Check for correct number of input arguments */
    if (nlhs != 2 || nrhs != 3)
        mexErrMsgTxt("Wrong number of arguments");
    
    /* Check that input arguments are sparse */
    if (mxIsSparse(prhs[1]) == false || mxIsSparse(prhs[2]) == false)
        mexErrMsgTxt("Input matrices must be sparse");
    
    /* Get pointers to input arguments */
    NP = *mxGetPr(prhs[0]);
    Ig = mxGetPr(prhs[1]);
    Ig_ir = mxGetIr(prhs[1]);
    Ig_jc = mxGetJc(prhs[1]);
    Ig_inv = mxGetPr(prhs[2]);
    Ig_inv_ir = mxGetIr(prhs[2]);
    Ig_inv_jc = mxGetJc(prhs[2]);
    
    /* Create output matrices */
    dims[0] = 3;
    dims[1] = 3;
    dims[2] = NP;
    plhs[0] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL); /* I */
    plhs[1] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL); /* I_inv */
    
    /* Get output pointers */
    I = mxGetPr(plhs[0]);
    I_inv = mxGetPr(plhs[1]);
    
    /* Main loop */
    for (p = 0; p < NP; p++)
    {
        /* Calculate I */
        for (i = Ig_jc[3*p]; i < Ig_jc[3*p+1]; i++)
            I[6*p + Ig_ir[i]] = Ig[i];
        for (i = Ig_jc[3*p+1]; i < Ig_jc[3*p+2]; i++)
            I[6*p+3+Ig_ir[i]] = Ig[i];
        for (i = Ig_jc[3*p+2]; i < Ig_jc[3*p+3]; i++)
            I[6*p+6+Ig_ir[i]] = Ig[i];
        
        /* Calculate I_inv */
        for (i = Ig_inv_jc[3*p]; i < Ig_inv_jc[3*p+1]; i++)
            I_inv[6*p + Ig_inv_ir[i]] = Ig_inv[i];
        for (i = Ig_inv_jc[3*p+1]; i < Ig_inv_jc[3*p+2]; i++)
            I_inv[6*p+3+Ig_inv_ir[i]] = Ig_inv[i];
        for (i = Ig_inv_jc[3*p+2]; i < Ig_inv_jc[3*p+3]; i++)
            I_inv[6*p+6+Ig_inv_ir[i]] = Ig_inv[i];
    }
}